/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is RPXP for Mozilla.
 *
 * The Developer of the Original Code is Rich Persaud
 * Portions created by the Initial Developer are Copyright (C) 2004-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Derived from "Adblock for Mozilla", Copyright (C) 2002 by Contributor(s):
 *
 * 	Henrik Aasted Sorensen
 *	Stefan Kinitz
 *	Wladimir Palant
 *	rue
 *
 * ***** END LICENSE BLOCK ***** */

var gIsLoaded; 		// Bool: did rpxp successfully load (set by rpxp.xul) -- global var.
var gIsEnabled; 	// Bool: is rpxp enabled (set by pref-observer) -- global var.

const ID_RPXP_BRANCH		=	'rpxp.dailykos.com.'		;

// Preferences observer object; implements nsIObserver
function rpxpPrefObserver() 
{
	try {
		var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		this.Branch = prefService.getBranch( ID_RPXP_BRANCH );
		
		this.windowInit = false;
		this.statusElement = null, this.statusText = null;
		this.obsrvRe = 	/^rpxp\.dailykos\.com\.(observer)/;
		this.mainRe = 	/^rpxp\.dailykos\.com\.(enabled)/;

		var pref = prefService.getBranch(null); // preferences root node
		this.pbi = pref.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
		this.pbi.addObserver( ID_RPXP_BRANCH, this, true); // keep this last, in case we hit an error.
	} catch(e) { alert("Rpxp: prefs-Observer failed to attach: " + e + "\n"); }
}

rpxpPrefObserver.prototype = 
{	
	// nsISupports interface implementation -- for weak-reference by pref-observer service
	QueryInterface: function(iid) 	
	{
		if (!iid.equals(Components.interfaces.nsISupports)
				&& !iid.equals(Components.interfaces.nsISupportsWeakReference)
				&& !iid.equals(Components.interfaces.nsIObserver)) {
			dump("Rpxp Window Pref-Observer factory object: QI unknown interface: " + iid + "\n");
			throw Components.results.NS_ERROR_NO_INTERFACE; }
		return this;
	},
	
	// removes the observer-object from service -- called when the window is no longer open
	removeObserver: function() 
	{
		this.pbi.removeObserver( ID_RPXP_BRANCH, this);
		delete this;
	},
	
	// observer-function
	/* subject: [wrapped nsISupports :: nsIPrefBranch], nsIPrefBranchInternal
	   topic: "changed"*/
	observe: function(subject, topic, prefName) 
	{
		// if we don't have a valid window (closed)
		if ( !(typeof(document) == 'object' && document) ) 
		{
			this.removeObserver(); 
			return; 
		} 
		
		// on window-init, OR a status-critical pref change
		if ((!this.windowInit && this.obsrvRe.test(prefName)) || this.mainRe.test(prefName)) 
		{
			// Create status-element pointers
			if (!this.windowInit) 
			{
				this.statusElement 	= document.getElementById("rpxp-status");
				this.statusText 	= document.getAnonymousNodes(this.statusElement)[1]; 
			}
			
			// Enabled / Disabled
			if ( !this.windowInit || prefName == ID_RPXP_BRANCH + "enabled") 
			{
				gIsEnabled = (!this.Branch.prefHasUserValue("enabled") 
								|| this.Branch.getBoolPref("enabled")); // default:true
				if (!gIsLoaded) 
				{ 
					this.statusText.value = "RPXP Not Loaded"; 
					this.statusElement.setAttribute("status", "unloaded"); 
				}
				else if (!gIsEnabled) 
				{ 
					this.statusText.value = "RPXP Off"; 
					this.statusElement.setAttribute("status", "notlistening"); 
				}
				else 
				{ 
					this.statusText.value = "RPXP On"; 	
					this.statusElement.setAttribute("status", "Enabled"); 	
				} 
			}
	
			// window-init
			if (!this.windowInit) 
			{
				// disable conflicting key-bindings
				var conflictingKeys, rpxpKeys = ["b", "p"];
				for (var n = 0 ; n < rpxpKeys.length ; n++) 
				{
					conflictingKeys = document.getElementsByAttribute("key", rpxpKeys[n]);
					for (var q = 0 ; q < conflictingKeys.length ; q++)
						if (!( /rpxp/.test(conflictingKeys[q].getAttribute("id")) ) && conflictingKeys[q].hasAttribute("modifiers"))
							if (/accel/.test(conflictingKeys[q].getAttribute("modifiers")) && /shift/.test(conflictingKeys[q].getAttribute("modifiers")) )
								conflictingKeys[q].parentNode.removeChild(conflictingKeys[q]);
				}
				
				// flag as initialized
				this.windowInit = true;
			}
    	}
	} // observe() 
}

// runs the pref-observer once, by changing a dummy-pref 
//  -- called by the status-element's binding 'onconstruct' (rpxp.xml)
function initRpxpPrefObserver() 
{
	try 
	{
		var prefObj = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var Branch = prefObj.getBranch( ID_RPXP_BRANCH );
		var tempPref = (Branch.prefHasUserValue("observer")) ? Branch.getCharPref("observer") : "-"; // default:'-'      :P

		switch (tempPref) {
			case "-": 
				Branch.setCharPref("observer", "*");
				break;
			case "*":
				Branch.setCharPref("observer", "-");
				break;
		}
	} catch(e) {}
}

// toggles the rpxp-enable pref 
function toggleRpxpEnable() 
{
	if (!gIsLoaded) 
		return;

	var prefObj, Branch;
	
	if (gIsEnabled)
		try {
			prefObj = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			Branch = prefObj.getBranch( ID_RPXP_BRANCH );
			Branch.setBoolPref("enabled", false);
		} 
		catch(e) { alert("Rpxp could not be disabled:\n\n" + e); }
	else
		try {
			prefObj = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			Branch = prefObj.getBranch( ID_RPXP_BRANCH );
			Branch.setBoolPref("enabled", true);
		} 
		catch(e) { alert("Rpxp could not be enabled:\n\n" + e); }
}

// reveals context menu-items as needed
function rpxpSetContext ( evt ) 
{
	if ( evt.target.id != 'contentAreaContextMenu' )
		return																	;

	var isUser, isBlue, isRed, isNeutral
		, target 		=	gContextMenu.target										
		, isLink		=	( target.nodeName.toLowerCase() == 'a' )				
		, isSelected	=	( gContextMenu.searchSelected().length != 0 )
	

	if ( isLink )
	{
		/* Note: FireFox has trailing slash */
		isUser		=  (   gContextMenu.link.href.match(/\/user\// ) 
						|| gContextMenu.link.href.match(/\/\/.*\.dailykos\.com\/?$/));

		/* const COLOR_BLUE = '#1E90FF'	;	// dodgerblue
		 * const COLOR_RED  = '#DC143C'	;	// crimson
		 * const COLOR_GREEN= '#228B22  ;	// forestgreen
		 */

		isBlue		=  ( gContextMenu.link.style.color == "rgb(30, 144, 255)" )	;
		isRed		=  ( gContextMenu.link.style.color == "rgb(220, 20, 60)" )	;
		isGreen		=  ( gContextMenu.link.style.color == "rgb(34, 139, 34)" )	;
		isNeutral	=  ( ! gContextMenu.link.style.color )						;
	}

	gContextMenu.showItem( 'rpxp-blue-menuitem', ( isLink && isUser && ( isRed  || isGreen || isNeutral )  ));
	gContextMenu.showItem( 'rpxp-red-menuitem',  ( isLink && isUser && ( isBlue || isGreen || isNeutral )  ));
	gContextMenu.showItem( 'rpxp-green-menuitem',  ( isLink && isUser && ( isBlue || isRed || isNeutral )  ));
	gContextMenu.showItem( 'rpxp-neutral-menuitem', ( isLink && isUser && ( isBlue || isRed || isGreen )  ));
	gContextMenu.showItem( 'rpxp-clipboard', isSelected );

} // rpxpSetContext()

// Context-menu Dispatchers
function rpxpSetBlue (isFrame, targetNode)
{
	rpxpSetMember ( isFrame, targetNode, 'blue' );
}

function rpxpSetRed (isFrame, targetNode)
{
	rpxpSetMember ( isFrame, targetNode, 'red' );
}

function rpxpSetGreen (isFrame, targetNode)
{
	rpxpSetMember ( isFrame, targetNode, 'green' );
}

function rpxpSetNeutral (isFrame, targetNode)
{
	rpxpSetMember ( isFrame, targetNode, 'neutral' );
}

// Context-menu Worker
function rpxpSetMember (isFrame, targetNode, pType) 
{
	if (typeof(targetNode) != 'undefined' && targetNode) var node = targetNode;
	else node = gContextMenu.target;

	if (   node.href.match(/\/user\// ) 
		|| node.href.match(/\/\/.*\.dailykos\.com\/?$/) )
	{
		try 
		{
			var _rpxp 	   = Components
								.classes["@mozilla.org/rpxp;1"]
								.getService().wrappedJSObject			;
			var _observer  = Components
								.classes["@mozilla.org/observer-service;1"]
								.getService(Components.interfaces.nsIObserverService);
		} 
		catch(e) { alert("rpxpGlobalObject could not be retrieved:\n\n" + e); }

		if   	( pType == 'blue' 	 ) _rpxp.fSetBlue		( node )	;
		else if ( pType == 'red' 	 ) _rpxp.fSetRed 		( node )	;
		else if ( pType == 'green' 	 ) _rpxp.fSetGreen 		( node )	;
		else if ( pType == 'neutral' ) _rpxp.fSetNeutral 	( node )	;
		else						   return							;

		_rpxp.fUpdateLinks( node.ownerDocument.links, node.text )		;
		_observer.notifyObservers( null, "Rpxp-SerializeMembers", null );
	}

} // rpxpSetMember()

// Context-click for selected text blockquoting in dKos Auto-format
function rpxpQuoteToClipboard (isFrame, targetNode)
{
    try
    {
        var clip =  Components
                    .classes["@mozilla.org/widget/clipboardhelper;1"]
                    .getService(Components.interfaces.nsIClipboardHelper);
    }
    catch(e) { alert("Clipboard interface not available:\n\n" + e); }

    var thisWin = window.content.document;

    clip.copyString ( '<a href="' + thisWin.location + '">'
                        + thisWin.location.host + "</a> :\n"
                        + '<div class="blockquote">'
                        + thisWin.getSelection() + '</div>' );
}

// Open the settings window.
function rpxpSettings() 
{
	var settingsHandle = window.open( "chrome://rpxp/content/settings.xul"
									, "rpxpPreferences"
									, "chrome,resizable,centerscreen,close=no" );
	settingsHandle.focus();
}

/* debug
function printObj ( testObj )    
{
    var szObj       =   ''                                      ;
    for ( var i in testObj )
	{
		if ( typeof( testObj[i] ) != 'function' ) 
			szObj       += "\nobj[" + i + ']=' + testObj[i]       ;
	}
    return szObj    ;
}

*/
